import 'dart:convert';

import 'package:bagel_db/bagel_db.dart';

import 'package:dio/dio.dart';
import 'package:random_string/random_string.dart';
import 'package:test/test.dart';

getItemsTest() {
  group('get items test', () {
    // given
    BagelDB bagelDB = BagelDB(
        organizationId: 'brh2tbi23akg00ei0t0g',
        apiKey: 'AIzaSyDfZNuVbEN_73RhORbwo-A2hRMMpKBOZZU',
        projectId: 'brh2tg223akg00ei0ung');
    BagelDBRequest bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');

    clear() {
      bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');
    }

    test('basic data fetching test', () async {
      // when
      Response response = await bagelDBRequest.get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Boruch"));
    });

    test('basic query test', () async {
      clear();
      // when
      Response response = await bagelDBRequest.addQuery('name:Dov').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Dov"));
    });

    test('multiple queries test', () async {
      clear();
      // when
      Response response = await bagelDBRequest.addQuery('position:developer').addQuery('age:>:27').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      print(data);
      expect(data[0]["name"], contains("Nati"));
    });

    test('sort test', () async {
      clear();
      // when
      Response response = await bagelDBRequest.addSort('age').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[1]["name"], contains("Maayan"));
    });

    test('descending sort test', () async {
      clear();
      // when
      Response response = await bagelDBRequest.addSort('age').addOrder('desc').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Dov"));
    });

    test('results per page test', () async {
      clear();
      // when
      Response response = await bagelDBRequest.addResponsesPerPage('2').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data.length, lessThan(3));
    });

    test('everything success test', () async {
      clear();
      Response response = await bagelDBRequest.everything().get();
      var data = response.data;
      expect(response.statusCode, equals(200));
      expect(data[0]["seniority"][0]["item"]["seniority"], equals(6));
    });

    test('everything fail test', () async {
      clear();
      BagelDBRequest bagelDBRequestFalse = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');
      Response falseResponse = await bagelDBRequestFalse.get();
      var falseData = falseResponse.data;
      expect(falseResponse.statusCode, equals(200));
      expect(falseData[0]["seniority"][0]["item"], equals(null));
    });

    test('doc test', () async {
      clear();
      Response response = await bagelDBRequest.addDoc('5ee22f010a6090a13e2f777a').get();
      var data = response.data;
      expect(response.statusCode, equals(200));
      print(data);
      expect(data[0], null);
      expect(data["name"], contains('Boruch'));
    });
  });
}

postItemTest() {
  group('post item test', () {
    // given
    BagelDB bagelDB = BagelDB(
        organizationId: 'brh2tbi23akg00ei0t0g',
        apiKey: 'AIzaSyDfZNuVbEN_73RhORbwo-A2hRMMpKBOZZU',
        projectId: 'brh2tg223akg00ei0ung');
    BagelDBRequest bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');

    test('post test', () async {
      // when
      Response response = await bagelDBRequest.post(item: {'name': 'Renana', 'age': 34, 'position': 'COO'});
      // then
      expect(response.statusCode, equals(201));
    });
  });
}

putItemTest() {
  group('put item test', () {
    // given
    BagelDB bagelDB = BagelDB(
        organizationId: 'brh2tbi23akg00ei0t0g',
        apiKey: 'AIzaSyDfZNuVbEN_73RhORbwo-A2hRMMpKBOZZU',
        projectId: 'brh2tg223akg00ei0ung');
    BagelDBRequest bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');

    test('put test', () async {
      // when
      Response response = await bagelDBRequest.put(
          item: {'name': 'Renana', 'age': 34, 'position': randomString(10), '_id': '5ee7661447f94e0c9b4afb75'},
          itemId: '5ee7661447f94e0c9b4afb75');
      // then
      expect(response.statusCode, lessThan(300));
    });
  });
}

deleteItemTest() {
  group('delete item test', () {
    // given
    BagelDB bagelDB = BagelDB(
        organizationId: 'brh2tbi23akg00ei0t0g',
        apiKey: 'AIzaSyDfZNuVbEN_73RhORbwo-A2hRMMpKBOZZU',
        projectId: 'brh2tg223akg00ei0ung');
    BagelDBRequest bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');

    test('delete test', () async {
      Response resp = await bagelDBRequest.post(item: {'name': 'Renana', 'age': 34, 'position': 'designer'});
      // then
      var res = resp.data;
      print(res);
      bagelDBRequest = BagelDBRequest(bagelDB: bagelDB, collection: 'testItems');
      // when
      Response response = await bagelDBRequest.delete(itemId: jsonDecode(res)["id"]);
      // then
      expect(response.statusCode, equals(200));
    });
  });
}

void main() {
  getItemsTest();
  postItemTest();
  putItemTest();
  deleteItemTest();
}
